# Configuration file for the Sphinx documentation builder.
#
# This file only contains a selection of the most common options. For a full
# list see the documentation:
# https://www.sphinx-doc.org/en/master/usage/configuration.html

# -- Path setup --------------------------------------------------------------

# If extensions (or modules to document with autodoc) are in another directory,
# add these directories to sys.path here. If the directory is relative to the
# documentation root, use os.path.abspath to make it absolute, like shown here.
#
import os
import sys

# from tktooltip import __version__

sys.path.insert(0, os.path.abspath(".."))


# -- Project information -----------------------------------------------------

project = "tkinter-tooltip"
copyright = "2021-, Giannis Nikiteas"
author = "Giannis Nikiteas"

# The full version, including alpha/beta/rc tags
# release = __version__


# -- General configuration ---------------------------------------------------

# Add any Sphinx extension module names here, as strings. They can be
# extensions coming with Sphinx (named 'sphinx.ext.*') or your custom
# ones.
extensions = [
    "sphinxarg.ext",
    "sphinx.ext.autodoc",
    "sphinx.ext.autosectionlabel",
    "sphinx.ext.autosummary",
    "sphinx.ext.napoleon",
    "sphinx.ext.intersphinx",
    "sphinx.ext.inheritance_diagram",
    "sphinx_autodoc_typehints",
    "sphinx.ext.autosectionlabel",
    "sphinx_design",
    "sphinx_copybutton",
    "myst_parser",
    "sphinx_sitemap",
]
# For sphinx_design in Markdown
myst_enable_extensions = ["colon_fence"]
suppress_warnings = ["myst.header"]

# Add any paths that contain templates here, relative to this directory.
templates_path = ["_templates"]
source_suffix = [".rst", ".md"]


# List of patterns, relative to source directory, that match files and
# directories to ignore when looking for source files.
# This pattern also affects html_static_path and html_extra_path.
exclude_patterns = ["_build", "Thumbs.db", ".DS_Store"]


# -- Options for HTML output -------------------------------------------------

# The theme to use for HTML and HTML Help pages.  See the documentation for
# a list of builtin themes.
#
html_theme = "alabaster"
html_theme = "furo"
html_title = "tkinter-tooltip"
# html_logo = "../assets/images/tooltip_logo.svg"
html_favicon = "../assets/images/tooltip_logo.svg"
html_baseurl = "https://gnikit.github.io/tkinter-tooltip/"

# Add any paths that contain custom static files (such as style sheets) here,
# relative to this directory. They are copied after the builtin static files,
# so a file named "default.css" will overwrite the builtin "default.css".
html_static_path = ["_static"]

# Add any extra paths that contain custom files (such as robots.txt or
# .htaccess) here, relative to this directory. These files are copied
# directly to the root of the documentation.
html_extra_path = ["html_extra"]
# Default is {version}{lang}{link}
sitemap_url_scheme = "{link}"

display_toc = True
# autodoc_default_flags = ["members"]
autosummary_generate = True


intersphinx_mapping = {
    "python": ("https://docs.python.org/3.10", None),
}

inheritance_graph_attrs = {
    "size": '"6.0, 8.0"',
    "fontsize": 32,
    "bgcolor": "transparent",
}
inheritance_node_attrs = {
    "color": "black",
    "fillcolor": "white",
    "style": '"filled,solid"',
}
inheritance_edge_attrs = {
    "penwidth": 1.2,
    "arrowsize": 0.8,
}



import re
import tkinter as tk
from pathlib import Path
from tkinter import ttk

import sv_ttk
from PIL import Image, ImageTk

from tktooltip import ToolTip


class App(ttk.Frame):
    def __init__(self, parent):
        ttk.Frame.__init__(self, parent)

        # Create widgets
        self.setup_widgets()

        # Bind the entries to the validator methods
        self.bind_them()

    def setup_widgets(self):
        self.color = "0,0,0"  # Default color to replace (it's black)
        img = Path(__file__).parent / "tooltip_logo.png"  # image name to loads
        self.img = Image.open(img).resize((100, 100))  # Load and scale image
        self.img_tk = ImageTk.PhotoImage(self.img)  # Store to stop garbage collection
        self.img_label = ttk.Label(self, image=self.img_tk)  # Create label with image
        ToolTip(self.img_label, msg=self.colorcode)  # Add toolip to label
        self.img_label.pack()

        # Create a mask with non-zero pixel coordinates
        self.mask = []
        for i in range(self.img_tk.width()):
            for j in range(self.img_tk.height()):
                if self.img.getpixel((i, j)) != (0, 0, 0, 0):
                    self.mask.append((i, j))

        self.color_entry = ttk.Entry(self)
        self.color_entry.pack(padx=50, pady=(25, 50))
        ToolTip(self.color_entry, msg="Input RGB values\nseparated by commas")

    def bind_them(self):
        self.color_entry.bind("<FocusOut>", self.validate_color)
        self.color_entry.bind("<FocusIn>", self.validate_color)
        self.color_entry.bind("<KeyRelease>", self.validate_color)

    def validate_color(self, *_):
        """Checks if color is valid RGB values"""
        if re.match(r"^\d{1,3},\d{1,3},\d{1,3}$", self.color_entry.get()):
            self.color_entry.state(["!invalid"])
        else:
            self.color_entry.state(["invalid"])

        if (
            "invalid" not in self.color_entry.state()
            and self.color != self.color_entry.get()
        ):
            self.color = self.color_entry.get()
            r, g, b = self.color_entry.get().split(",")
            # Using the cached mask replace the all the non-zero pixels
            for i, j in self.mask:
                self.img.putpixel((i, j), (int(r), int(g), int(b)))
            self.img_tk = ImageTk.PhotoImage(self.img)
            self.img_label.config(image=self.img_tk)

    def colorcode(self):
        """Returns the RGB and HEX color codes of a color"""
        msg = "RGB: ({},{},{})".format(*self.color.split(","))
        msg = msg + "\nHEX: #{:02x}{:02x}{:02x}".format(
            *list(map(int, self.color.split(",")))
        )
        return msg


def main():
    root = tk.Tk()
    root.title("")
    sv_ttk.set_theme("dark")

    app = App(root)
    app.pack(fill="both", expand=True)
    root.update_idletasks()  # Make sure every screen redrawing is done

    width, height = root.winfo_width(), root.winfo_height()
    x = int((root.winfo_screenwidth() / 2) - (width / 2))
    y = int((root.winfo_screenheight() / 2) - (height / 2))
    # Set a minsize for the window, and place it in the middle
    root.minsize(width, height)
    root.geometry(f"+{x}+{y}")
    root.mainloop()


if __name__ == "__main__":
    main()



import time
import tkinter as tk
import tkinter.ttk as ttk
from itertools import product
from tkinter import font as tkFont

from tktooltip import ToolTip


def custom_font(frame, **kwargs):
    return tkFont.Font(frame, **kwargs)


def main():
    root = tk.Tk()
    s = ttk.Style()
    s.configure("custom.TButton", foreground="#208020", background="#fafafa")
    # root.tk.call("source", "themes/sun-valley/sun-valley.tcl")
    # root.tk.call("set_theme", "dark")
    btn_list = []
    for i, j in list(product(range(2), range(4))):
        text = f"delay={i}s\n"
        delay = i
        if j >= 2:
            follow = True
            text += "follow tooltip: \u2714\n"  # yes
        else:
            follow = False
            text += "follow tooltip: \u274C\n"  # no
        if j % 2 == 0:
            msg = time.asctime
            text += "tooltip: Function"
        else:
            msg = f"Button at {str((i, j))}"
            text += "tooltip: String"
        btn_list.append(ttk.Button(root, text=text, style="custom.TButton"))
        ToolTip(
            btn_list[-1],
            msg=msg,
            follow=follow,
            delay=delay,
            # Parent frame arguments
            parent_kwargs={"bg": "black", "padx": 5, "pady": 5},
            # These are arguments to the tk.Message
            fg="#202020",
            bg="#fafafa",
            padx=10,
            pady=10,
            font=custom_font(root, size=8, weight=tkFont.BOLD),
        )
        btn_list[-1].grid(row=i, column=j, sticky="nsew", ipadx=20, ipady=20)
    root.mainloop()


if __name__ == "__main__":
    main()



import tkinter as tk
from typing import Callable

import pytest

from tktooltip import ToolTip, ToolTipStatus


@pytest.fixture()
def widget():
    widget = tk.Entry()
    widget.pack()
    widget.update()
    yield widget
    widget.destroy()


def test_tooltip(widget: tk.Widget):
    tooltip = ToolTip(
        widget,
        msg="Test Tooltip",
        delay=0.5,
        follow=False,
        refresh=0.5,
        x_offset=20,
        y_offset=20,
        parent_kwargs={"bg": "red", "padx": 2, "pady": 2},
        fg="white",
        bg="red",
        font=("Helvetica", 12),
    )

    assert tooltip.msg == "Test Tooltip"
    assert tooltip.delay == 0.5
    assert not tooltip.follow
    assert tooltip.refresh == 0.5
    assert tooltip.x_offset == 20
    assert tooltip.y_offset == 20


@pytest.mark.parametrize(
    "msg",
    [
        1,
        (["text 1", 2]),
    ],
)
def test_tooltip_exceptions(
    widget: tk.Widget,
    msg: str | list[str] | Callable[[], str | list[str]],
):
    with pytest.raises(TypeError):
        ToolTip(widget, msg=msg)  # type: ignore


def test_tooltip_follow(widget: tk.Widget):
    tooltip = ToolTip(widget, msg="Test", follow=False)
    widget.event_generate("<Enter>")
    assert not tooltip.follow


@pytest.mark.parametrize(
    "msg",
    [
        (lambda: "Callable Tooltip"),
        (lambda: ["Callable Tooltip 1", "Callable Tooltip 2"]),
        "text",
        (["text 1", "text 2"]),
    ],
)
def test_tooltip_show(
    widget: tk.Widget,
    msg: str | list[str] | Callable[[], str | list[str]],
):
    tooltip = ToolTip(widget, msg=msg)
    assert tooltip.status == ToolTipStatus.OUTSIDE
    widget.event_generate("<Enter>")
    assert tooltip.status == ToolTipStatus.INSIDE
    tooltip._show()
    assert tooltip.status == ToolTipStatus.VISIBLE
    widget.event_generate("<Leave>")
    assert tooltip.status == ToolTipStatus.OUTSIDE


def test_tooltip_destroy(widget: tk.Widget):
    tooltip = ToolTip(widget, msg="Test")
    widget.event_generate("<Enter>")
    assert tooltip.status == ToolTipStatus.INSIDE
    tooltip._show()
    assert tooltip.status == ToolTipStatus.VISIBLE
    tooltip.destroy()
    print(tooltip.bindigs)
    assert tooltip.bindigs == []



import tkinter as tk

from tktooltip import ToolTip  # Assuming tooltip.py is in the same directory


class Application(tk.Tk):
    def __init__(self):
        super().__init__()
        self.iter = 0

        # Create a label with a tooltip
        self.label = tk.Label(self, text="Hover over me!")
        self.label.pack(padx=10, pady=10)
        self.tooltip = ToolTip(self.label, msg=f"Hello {self.iter}")

        # Create a button that destroys the tooltip
        self.destroy_b = tk.Button(
            self, text="Destroy Tooltip", command=self.destroy_tooltip
        )
        self.destroy_b.pack(padx=10, pady=10)
        self.create_b = tk.Button(
            self, text="Create Tooltip", command=self.create_tooltip
        )
        self.create_b.pack(padx=10, pady=10)

    def destroy_tooltip(self):
        # Destroy the tooltip
        if self.tooltip is not None:
            self.tooltip.destroy()
            self.tooltip = None

    def create_tooltip(self):
        # Create the tooltip
        print(type(self.tooltip))
        print(self.tooltip)
        print("===========")
        for c in list(self.children.values()):
            print(c)
        print("***********")
        if self.tooltip is None:
            self.iter += 1
            self.tooltip = ToolTip(self.label, msg=f"Hello {self.iter}")


if __name__ == "__main__":
    app = Application()
    app.mainloop()



try:
    from importlib.metadata import PackageNotFoundError, version
except ModuleNotFoundError:
    from importlib_metadata import PackageNotFoundError, version
try:
    __version__ = version("tkinter-tooltip")
except PackageNotFoundError:
    from setuptools_scm import get_version

    __version__ = get_version(root="..", relative_to=__file__)



"""
Module defining the ToolTip widget
"""

from __future__ import annotations

import time
import tkinter as tk
from contextlib import suppress
from enum import Enum, auto
from typing import Any, Callable

# This code is based on Tucker Beck's implementation licensed under an MIT License
# Original code: http://code.activestate.com/recipes/576688-tooltip-for-tkinter/


class ToolTipStatus(Enum):
    OUTSIDE = auto()
    INSIDE = auto()
    VISIBLE = auto()


class Binding:
    def __init__(self, widget: tk.Widget, binding_name: str, functor: Callable) -> None:
        self._widget = widget
        self._name: str = binding_name
        self._id: str = self._widget.bind(binding_name, functor, add="+")

    def unbind(self) -> None:
        self._widget.unbind(self._name, self._id)


class ToolTip(tk.Toplevel):
    """
    Creates a ToolTip (pop-up) widget for tkinter
    """

    DEFAULT_PARENT_KWARGS = {"bg": "black", "padx": 1, "pady": 1}
    DEFAULT_MESSAGE_KWARGS = {"aspect": 1000}
    S_TO_MS = 1000

    def __init__(
        self,
        widget: tk.Widget,
        msg: str | list[str] | Callable[[], str | list[str]],
        delay: float = 0.0,
        follow: bool = True,
        refresh: float = 1.0,
        x_offset: int = +10,
        y_offset: int = +10,
        parent_kwargs: dict | None = None,
        **message_kwargs: Any,
    ):
        """Create a ToolTip. Allows for `**kwargs` to be passed on both
            the parent frame and the ToolTip message

        Parameters
        ----------
        widget : tk.Widget
            The widget this ToolTip is assigned to
        msg : `Union[str, Callable]`, optional
            A string message (can be dynamic) assigned to the ToolTip.
            Alternatively, it can be set to a function thatreturns a string,
            by default None
        delay : `float`, optional
            Delay in seconds before the ToolTip appears, by default 0.0
        follow : `bool`, optional
            ToolTip follows motion, otherwise hides, by default True
        refresh : `float`, optional
            Refresh rate in seconds for strings and functions when mouse is
            stationary and inside the widget, by default 1.0
        x_offset : `int`, optional
            x-coordinate offset for the ToolTip, by default +10
        y_offset : `int`, optional
            y-coordinate offset for the ToolTip, by default +10
        parent_kwargs : `dict`, optional
            Optional kwargs to be passed into the parent frame,
            by default `{"bg": "black", "padx": 1, "pady": 1}`
        **message_kwargs : tkinter `**kwargs` passed directly into the ToolTip
        """
        self.widget = widget
        # ToolTip should have the same parent as the widget unless stated
        # otherwise in the `parent_kwargs`
        tk.Toplevel.__init__(self, **(parent_kwargs or self.DEFAULT_PARENT_KWARGS))
        self.withdraw()  # Hide initially in case there is a delay
        # Disable ToolTip's title bar
        self.overrideredirect(True)

        # StringVar instance for msg string|function
        self.msg_var = tk.StringVar()
        self.msg = msg
        self._update_message()
        self.delay = delay
        self.follow = follow
        self.refresh = refresh
        self.x_offset = x_offset
        self.y_offset = y_offset
        # visibility status of the ToolTip inside|outside|visible
        self.status = ToolTipStatus.OUTSIDE
        self.last_moved = 0.0
        # use Message widget to host ToolTip
        self.message_kwargs: dict = self.DEFAULT_MESSAGE_KWARGS.copy()
        self.message_kwargs.update(message_kwargs)
        self.message_widget = tk.Message(
            self,
            textvariable=self.msg_var,
            **self.message_kwargs,
        )
        self.message_widget.grid()
        self.bindigs = self._init_bindings()

    def _init_bindings(self) -> list[Binding]:
        """Initialize the bindings."""
        bindings = [
            Binding(self.widget, "<Enter>", self.on_enter),
            Binding(self.widget, "<Leave>", self.on_leave),
            Binding(self.widget, "<ButtonPress>", self.on_leave),
        ]
        if self.follow:
            bindings.append(
                Binding(self.widget, "<Motion>", self._update_tooltip_coords)
            )
        return bindings

    def destroy(self) -> None:
        """Destroy the ToolTip and unbind all the bindings."""
        with suppress(tk.TclError):
            for b in self.bindigs:
                b.unbind()
            self.bindigs.clear()
            super().destroy()

    def on_enter(self, event: tk.Event) -> None:
        """
        Processes motion within the widget including entering and moving.
        """
        self.last_moved = time.perf_counter()
        self.status = ToolTipStatus.INSIDE
        self._update_tooltip_coords(event)
        self.after(int(self.delay * self.S_TO_MS), self._show)

    def on_leave(self, event: tk.Event | None = None) -> None:
        """
        Hides the ToolTip.
        """
        self.status = ToolTipStatus.OUTSIDE
        self.withdraw()

    def _update_tooltip_coords(self, event: tk.Event) -> None:
        """
        Updates the ToolTip's position.
        """
        self.geometry(f"+{event.x_root + self.x_offset}+{event.y_root + self.y_offset}")

    def _update_message(self) -> None:
        """Update the message displayed in the tooltip."""
        if callable(self.msg):
            msg = self.msg()
            if isinstance(msg, list):
                msg = "\n".join(msg)
        elif isinstance(self.msg, str):
            msg = self.msg
        elif isinstance(self.msg, list):
            msg = "\n".join(self.msg)
        else:
            raise TypeError(
                f"ToolTip `msg` must be a string, list of strings, or a "
                f"callable returning them, not {type(self.msg)}."
            )
        self.msg_var.set(msg)

    def _show(self) -> None:
        """
        Displays the ToolTip.

        Recursively queues `_show` in the scheduler every `self.refresh` seconds
        """
        if (
            self.status == ToolTipStatus.INSIDE
            and time.perf_counter() - self.last_moved >= self.delay
        ):
            self.status = ToolTipStatus.VISIBLE

        if self.status == ToolTipStatus.VISIBLE:
            self._update_message()
            self.deiconify()

            # Recursively call _show to update ToolTip with the newest value of msg
            # This is a race condition which only exits when upon a binding change
            # that in turn changes the `status` to outside
            self.after(int(self.refresh * self.S_TO_MS), self._show)



"""
TkToolTip
=========

Provides a tooltip (pop-up) widget for tkinter

Features:
---------
    - Normal tooltips
    - Functions tooltips
    - Delayed tooltips
    - Tracking tooltips
    - Theme-aware tooltips and fully customisable
"""

from .tooltip import ToolTip, ToolTipStatus
from .version import __version__

__all__ = [
    "ToolTip",
    "ToolTipStatus",
    "__version__",
]
